using System.ComponentModel;
using System.Reflection;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.GUI.MainScreen.Interfaces;
using Luski.GUI.MainScreen.UI;
using Luski.GUI.StartPage.UI;
using Luski.net;
using Luski.net.Enums;
using Luski.net.Interfaces;
using Luski.net.JsonTypes;
using OpenTK.Mathematics;
using OpenTK.Windowing.Desktop;
using OpenTK.Windowing.Common;
using Window = GraphicsManager.Window;

namespace Luski.GUI.MainScreen;

public class MainScreen : Window
{
    private static readonly NativeWindowSettings Settings = new()
    {
        Title = "Luski Login",
        WindowBorder = WindowBorder.Fixed,
        APIVersion = new Version(3, 2),
        StartFocused = true,
        Size = new OpenTK.Mathematics.Vector2i(481, 838),
    };

    private TabControl tc;
    private FlowLayout? channelpicker, friends;
    private RoundedButton? FriendManagerBtn;
    private Chat? chat;
    
    public MainScreen() : base(Settings)
    {
        Login login;
        Controls.Add(login = new Login());
        login.ChangeToApp += LoginOnChangeToApp;
        Thread t = new(_ => Encryption.GenerateKeys());
        t.Start();
    }

    public void AddGroup(SocketGroupChannel group)
    {
        Group friend = new(group);
        friend.ClickCon += ChannelOnClickCon;
        channelpicker!.Controls.Add(friend);
        chans.Add(friend);
    }

    private readonly List<Friend> fr = new();
    private List<IChannelPick> chans = new();
    public void AddFriend(SocketRemoteUser user)
    {
        Friend friend2 = new(user);
        friend2.ClickCon += ChannelOnClickCon;
        friends.Controls.Add(friend2);
        if (user.Channel is not null)
        {
            Friend friend = new(user);
            friend.ClickCon += ChannelOnClickCon;
            fr.Add(friend);
            channelpicker!.Controls.Add(friend);
            chans.Add(friend);
        }
    }

    private Task ChannelOnClickCon(IChannelPick arg)
    {
        if (!chat.Visible)
        {
            chat.Visible = true;
            tc.Visible = false;
        }
        chat!.UpdateTitle(arg);
        chat.Clear();
        IReadOnlyList<SocketMessage> messages = arg.Channel.GetMessages(200).Result;
        foreach (SocketMessage message in messages.Reverse())
        {
            chat.AddMessage(message);
        }
        return Task.CompletedTask;
    }

    private Task LoginOnChangeToApp()
    {
        Controls.Clear();
        Title = "Luski";
        Size = new(2048, 1334);
        WindowBorder = WindowBorder.Resizable;
        BackgroundColor = new Color4(34, 34, 34, 255);
        Controls.Add(new Rectangle(new Texture(Globals.Luski.CurrentUser.GetAvatar().Result)) { Anchor = ObjectAnchor.Bottom | ObjectAnchor.Left, Size = new(70, 70), Location = new(94, 1248)});
        Controls.Add(new Label(){ Anchor = ObjectAnchor.Bottom | ObjectAnchor.Left, Location = new(172, 1271), Text = Globals.Luski.CurrentUser.Username});
        FlowLayout ser;
        Controls.Add(chat = new() {Location = new(528,0)});
        Controls.Add(ser = new FlowLayout()
        {
            BackgroundColor = new(26, 26, 26, 255),
            Size = new(80,1334),
            Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom,
            
        });
        Controls.Add(tc = new()// this is here to showcase my good friend TCLL
        {
            Location = chat.Location,
            Size = chat.Size,
            BackgroundColor = chat.BackgroundColor,
            Anchor = ObjectAnchor.All,
            Visible = false
        });
        UserControl f;
        tc.AddPage("Frineds", f= new UserControl());
        tc.AddPage("Friend Request", new UserControl());
        tc.AddPage("Add Friend", new UserControl());
        f.Controls.Add(friends = new FlowLayout()
        {
            Size = f.Size,
            BackgroundColor = new(45,45,45,255),
            Anchor = ObjectAnchor.All
        });
        ser.Controls.Add(new Rectangle(Globals.LuskiTexture)
        {
            Size = new(80,80),
        });
        
        Controls.Add(channelpicker = new FlowLayout()
        {
            BackgroundColor = new(34,34,34,255),
            Size = new(448, 1232),
            Location = new(80, 0),
            Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom
        });
        channelpicker.Controls.Add(FriendManagerBtn = new RoundedButton() {Text = "Friends", Size = new(50, 80), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan});
        FriendManagerBtn.Clicked += FriendManagerBtnOnClicked;
        foreach (SocketGroupChannel ch in Globals.Luski.CurrentUser.Channels.Where(s => s is SocketGroupChannel).Cast<SocketGroupChannel>())
        {
            AddGroup(ch);
        }
        foreach (SocketRemoteUser item in Globals.Luski.CurrentUser.Friends)
        {
            if (item.Channel is not null) AddFriend(item);
        }
        
        SocketTextChannel chan = Globals.Luski.GetChannel<SocketTextChannel>(Globals.Luski.CurrentUser.SelectedChannel).Result;
        chat.UpdateTitle(chans.First(s => s.Channel.Id == chan.Id));
        IReadOnlyList<SocketMessage> messages = chan.GetMessages(200).Result;
        foreach (SocketMessage message in messages.Reverse())
        {
            chat.AddMessage(message);
        }
        Globals.Luski.UserStatusUpdate += LuskiOnUserStatusUpdate;
        DrawFrame();
        return Task.CompletedTask;
    }

    private Task LuskiOnUserStatusUpdate(IUser before, IUser After)
    {
        if (before is not SocketRemoteUser Before || Before.FriendStatus != FriendStatus.Friends) return Task.CompletedTask;
        Label stat = fr.Where(s => s.User.Id == before.Id).First()!.Status;
        Invoke(new Action(() =>
        {
            stat.Text = After.Status.ToString();
        }));
        return Task.CompletedTask;
    }
    
    private Task FriendManagerBtnOnClicked(IRenderObject arg)
    {
        if (chat.Visible)
        {
            chat.Visible = false;
            tc.Visible = true;
        }
        else
        {
            chat.Visible = true;
            tc.Visible = false;
        }
        return Task.CompletedTask;
    }
}